/******************************************************************************
 * $Header: /CommonBe/agmsmith/Programming/ScriptOCron/RCS/Trash.cpp,v 1.1 2015/12/21 00:12:26 agmsmith Exp $
 *
 * This code is from the [un]trash command for Haiku OS, as of December 2015.
 * Copyright (c) 2004, Francois Revol - revol@free.fr
 * provided under the MIT licence
 */

#include "Trash.h"

#include <stdio.h>
#include <string.h>

#include <Directory.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <KernelKit.h>
#include <Node.h>
#include <Path.h>

static const char kAttrOriginalPath [] = "_trk/original_path";

status_t trash (const char *f)
{
  status_t err;
  attr_info ai;
  dev_t dev = -1;
  int nr;
  const char *original_path;
  char trash_dir[B_PATH_NAME_LENGTH];
  char trashed_file[B_PATH_NAME_LENGTH];
  dev = dev_for_path(f);
  err = find_directory(B_TRASH_DIRECTORY, dev, false, trash_dir, B_PATH_NAME_LENGTH);
  if (err < 0)
    return err;
  BNode node(f);
  err = node.InitCheck();
  if (err < 0)
    return err;
  err = node.GetAttrInfo(kAttrOriginalPath, &ai);
  if (err == B_OK)
    return EALREADY;
  if (!strncmp(f, trash_dir, strlen(trash_dir)))
    return EALREADY;
  entry_ref er;
  err = get_ref_for_path(f, &er);
  BPath orgPath(&er);
  err = orgPath.InitCheck();
  if (err < 0)
    return err;
  original_path = orgPath.Path();
  BDirectory trashDir(trash_dir);
  err = trashDir.InitCheck();
  if (err < 0)
    return err;
  for (nr = 0; ; nr++) {
    if (nr > INT_MAX - 1)
      return B_ERROR;
    if (nr)
      snprintf(trashed_file, B_PATH_NAME_LENGTH-1, "%s/%s %d", trash_dir, er.name, nr);
    else
      snprintf(trashed_file, B_PATH_NAME_LENGTH-1, "%s/%s", trash_dir, er.name);
    if (!trashDir.Contains(trashed_file))
      break;
  }
  err = rename(original_path, trashed_file);
  if (err < 0)
    return err;
  
  err = node.WriteAttr(kAttrOriginalPath, B_STRING_TYPE, 0LL, original_path, strlen(original_path)+1);
  if (err < 0)
    return err;
  return 0;
}

/******************************************************************************
 * $Log: Trash.cpp,v $
 * Revision 1.1  2015/12/21 00:12:26  agmsmith
 * Initial revision
 *
 */
 